# SSL/TLS Configuration Guidance

## Summary

* Use TLS Everywhere
* Use a Strong TLS Protocol
* Use Strong Cipher Suites
* Do Not Disable Certificate Validation
* Use an Up to Date Implementation

Review the SSL Labs [SSL and TLS Deployment Best Practices](https://github.com/ssllabs/research/wiki/SSL-and-TLS-Deployment-Best-Practices)
document for additional information and best practices.

## Details

### Use TLS Everywhere

*TLS Everywhere* is a monicker the really means "encrypt all network
traffic whenever it crosses a trust boundary". A trust boundary is just
a fancy phrase meaning, "where you trust one side differently than the other".
However, like all things, there's some grey-area in here.

Common scenarios that definitely cross a trust bounary include:

* Connecting to a location on the Internet.
* Connecting to a location on another service, owned by another team at
  your company.

Scenarios where you almost certainly don't cross a trust bounary include:

* Connecting to a service running on the same operating system.

Scenarios somewhere in the middle include:

* Connections between containers, even on the same host.
* Connections between services within a virtual network, your own
  hardware rack, or through a VPN.

Its up to you to decice what your trust model looks like, but we recommend
encrypting network traffic as much as possible.

HTTPS (TLS over HTTP) is the most common application-layer communication
protocol, and most technology stacks have good support for TLS "out of the
box", but other common protocols, such as database connection strings,
almost always support encryption. For more information on database
connection strings, see [The Connection Strings Reference](https://www.connectionstrings.com/).

Even when using TLS, there are multiple configuration options that can make
the difference between a strong and a weak connection. These options usually
come with a default, and are often configurable in some way.

### Use a Strong TLS Protocol

First, there are multiple versions of the SSL/TLS protocol. As of March
2020, their security posture is generally agreed to be:

* SSLv2 - Broken, do not use under any circumstances.
* SSLv3 - Broken, do not use under any circumstances.
* TLSv1 - Weak, strongly consider disabling unless absolutely required.
* TLSv1.1 - Weak, strongly consider disabling unless absolutely required.
* TLSv1.2 - Strong, acceptable for use for the foreseeable future.
* TLSv1.3 - Strong, acceptable for use for the foreseeable future.

Generally speaking, you should target TLS 1.2 and TLS 1.3 on all clients
and servers.

### Use Strong Cipher Suites

Next to protocol version, the choice is cipher suite is the most important
aspect of a secure TLS configuration. Cipher suites are collections of
algorithms and protocols for key exchange, bulk data encryption, and
integrity validation.

A cipher suite is considered "strong" when it provides at least 128-bits
of security, forward secrecy, and strong authentication and key exchange
methods. These should be at the top of your list.

```
TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256
TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384
TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256
TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384
```

### Do Not Disable Certificate Validation

Certificate validation can be complex, but is fundamental to secure
communication. Software developers are often tempted to disable
certificate validation due to mis-configurations, but disabling 
validation is very nearly the same as disabling encryption altogether.

### Use an Up to Date Implementation

All software contains bugs, and that includes software that implements
cryptography. Strong algorithms and protocols can be undermined by code
defects, and as a result, you should keep your implementations up
to date.

## Severity Considerations

The severity of this finding should be increased if the underlying security
context is highly sensitive or if the network has low trust.

## References

* [Microsoft SDL Cryptographic Recommendations](http://download.microsoft.com/download/6/3/A/63AFA3DF-BB84-4B38-8704-B27605B99DA7/Microsoft%20SDL%20Cryptographic%20Recommendations.pdf)
* [SSL Labs: SSL and TLS Deployment Best Practices](https://github.com/ssllabs/research/wiki/SSL-and-TLS-Deployment-Best-Practices)
* [OWASP: Transport Layer Protection Cheat Sheet](https://owasp.org/www-project-cheat-sheets/cheatsheets/Transport_Layer_Protection_Cheat_Sheet)
* [OWASP: TLS Cipher String Cheat Sheet](https://owasp.org/www-project-cheat-sheets/cheatsheets/TLS_Cipher_String_Cheat_Sheet.html)

